@@ -26,6 +26,7 @@
 #include <set>
 #include <string>
 #include <climits>
+#include <vector>
 
 #if defined(HAVE_EXT_HASH_MAP)
 # include <ext/hash_map>
@@ -214,6 +215,8 @@ namespace pan
 
   typedef std::set<Quark> quarks_t;
 
+  typedef std::vector<Quark> quarks_v;
+
   /**
    * StrictWeakOrdering which sorts Quarks alphabetically.
    * This should be used sparingly, as it defeats Quark's main speed advantage.
