@@ -39,6 +39,7 @@ extern "C" {
 #include <pan/usenet-utils/mime-utils.h>
 #include <pan/data/data.h>
 #include <pan/tasks/task-post.h>
+#include "e-charset-picker.h"
 #include "pad.h"
 #include "hig.h"
 #include "post-ui.h"
@@ -149,14 +150,23 @@ PostUI :: get_body () const
   return body;
 }
 
+void
+PostUI :: set_always_run_editor (bool run)
+{
+  _prefs.set_flag ("always-run-editor", run);
+}
+
 void
 PostUI :: set_wrap_mode (bool wrap)
 {
-  const std::string s (get_body());
-  gtk_text_buffer_set_text (_body_buf, s.c_str(), s.size());
+  _prefs.set_flag ("compose-wrap-enabled", wrap);
 
-  gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(_body_view),
-                               wrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
+  if (_body_buf) {
+    const std::string s (get_body());
+    gtk_text_buffer_set_text (_body_buf, s.c_str(), s.size());
+    gtk_text_view_set_wrap_mode (GTK_TEXT_VIEW(_body_view),
+                                 wrap ? GTK_WRAP_WORD : GTK_WRAP_NONE);
+  }
 }
 
 /***
@@ -172,32 +182,24 @@ namespace
   void do_paste    (GtkAction * w, gpointer p) { g_signal_emit_by_name (get_focus(p), "paste_clipboard"); }
   void do_rot13    (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->rot13_selection(); }
   void do_edit     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->spawn_editor (); }
-  void do_editors  (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->manage_editors (); }
   void do_profiles (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->manage_profiles (); }
   void do_send     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->send_now (); }
   void do_save     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->save_draft (); }
   void do_open     (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->open_draft (); }
+  void do_charset  (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->prompt_for_charset (); }
   void do_close    (GtkAction * w, gpointer p) { static_cast<PostUI*>(p)->close_window (); }
   void do_wrap     (GtkToggleAction * w, gpointer p) { static_cast<PostUI*>(p)->set_wrap_mode (gtk_toggle_action_get_active (w)); }
-
-  void editor_selected_cb (GtkRadioAction * action, GtkRadioAction * current, gpointer post_gpointer)
-  {
-    if (action == current)
-    {
-      const char * command = (const char*) g_object_get_data (G_OBJECT(action), "editor-command");
-      static_cast<PostUI*>(post_gpointer)->set_editor_command (command);
-    }
-  }
+  void do_edit2    (GtkToggleAction * w, gpointer p) { static_cast<PostUI*>(p)->set_always_run_editor (gtk_toggle_action_get_active (w)); }
 
   GtkActionEntry entries[] =
   {
     { "file-menu", NULL, N_("_File") },
     { "edit-menu", NULL, N_("_Edit") },
     { "profile-menu", NULL, N_("_Profile") },
-    { "charsets-menu", NULL, N_("Set Character Encoding") },
     { "editors-menu", NULL, N_("Set Editor") },
     { "post-toolbar", NULL, "post" },
     { "post-article", GTK_STOCK_EXECUTE, N_("_Send Article"), "<control>Return", N_("Send Article Now"), G_CALLBACK(do_send) },
+    { "set-charset", NULL, N_("Set Character _Encoding..."), NULL, NULL, G_CALLBACK(do_charset) },
     { "save-draft", GTK_STOCK_SAVE, N_("Sa_ve Draft"), "<control>s", N_("Save as a Draft for Future Posting"), G_CALLBACK(do_save) },
     { "open-draft", GTK_STOCK_OPEN, N_("_Open Draft..."), "<control>o", N_("Open an Article Draft"), G_CALLBACK(do_open) },
     { "close", GTK_STOCK_CLOSE, NULL, NULL, NULL, G_CALLBACK(do_close) },
@@ -206,14 +208,14 @@ namespace
     { "paste", GTK_STOCK_PASTE, NULL, NULL, NULL, G_CALLBACK(do_paste) },
     { "rot13", GTK_STOCK_REFRESH, N_("_Rot13"), NULL, N_("Rot13 Selected Text"), G_CALLBACK(do_rot13) },
     { "run-editor", GTK_STOCK_JUMP_TO, N_("Run _Editor"), "<control>e", NULL, G_CALLBACK(do_edit) },
-    { "manage-editors", NULL, N_("_Manage Editor List..."), NULL, NULL, G_CALLBACK(do_editors) },
-    { "manage-profiles", NULL, N_("Manage Posting Pr_ofiles..."), NULL, NULL, G_CALLBACK(do_profiles) }
+    { "manage-profiles", GTK_STOCK_EDIT, N_("Edit P_osting Profiles"), NULL, NULL, G_CALLBACK(do_profiles) }
   };
 
   GtkToggleActionEntry toggle_entries[] =
   {
     { "wrap", GTK_STOCK_JUSTIFY_FILL, N_("_Wrap Text"), NULL, NULL, G_CALLBACK(do_wrap), true },
-    { "remember-charset", NULL, N_("Remember _Charset for This Group"), NULL, NULL, G_CALLBACK(on_remember_charset_toggled), true },
+    { "always-run-editor", NULL, N_("Always Run Editor"), NULL, NULL, G_CALLBACK(do_edit2), false },
+    { "remember-charset", NULL, N_("Remember Character Encoding for this Group"), NULL, NULL, G_CALLBACK(on_remember_charset_toggled), true },
     { "spellcheck", NULL, N_("Check _Spelling"), NULL, NULL, G_CALLBACK(on_spellcheck_toggled), true }
   };
 
@@ -228,146 +230,22 @@ namespace
     }
     gtk_box_pack_start (GTK_BOX(vbox), widget, false, false, 0);
   }
-
-  bool app_is_toggling_charset_action (false);
-}
-
-void
-PostUI :: charset_selected_cb_static (GtkRadioAction * action, GtkRadioAction * current, gpointer ui_gpointer)
-{
-  if (action == current)
-  {
-    const char * charset = (const char*) g_object_get_data (G_OBJECT(action), "charset");
-    static_cast<PostUI*>(ui_gpointer)->charset_selected_cb (charset);
-  }
-}
-
-void
-PostUI :: charset_selected_cb (const char * charset)
-{
-  // since user has specified a charset manually,
-  // we need to stop implicitly picking one
-  // whenever the newsgroups field changes.
-  if (_group_entry_changed_id && !app_is_toggling_charset_action) {
-    g_signal_handler_disconnect (_groups_entry, _group_entry_changed_id);
-    _group_entry_changed_id = 0;
-    if (_group_entry_changed_idle_tag) {
-      g_source_remove (_group_entry_changed_idle_tag);
-      _group_entry_changed_idle_tag = 0;
-    }
-  }
-
-  // set the charset
-  set_charset (charset);
 }
 
 #define DEFAULT_CHARSET  "UTF-8"
 
 void
-PostUI :: add_charset_list ()
+PostUI :: prompt_for_charset ()
 {
-  const struct CharsetStruct {
-    const char *charset, *name;
-  } charsets[] = {
-    {"ISO-8859-4",   N_("Baltic")},
-    {"ISO-8859-13",  N_("Baltic")},
-    {"windows-1257", N_("Baltic")},
-    {"ISO-8859-2",   N_("Central European")},
-    {"windows-1250", N_("Central European")},
-    {"gb2312",       N_("Chinese Simplified")},
-    {"big5",         N_("Chinese Traditional")},
-    {"ISO-8859-5",   N_("Cyrillic")},
-    {"windows-1251", N_("Cyrillic")},
-    {"KOI8-R",       N_("Cyrillic")},
-    {"KOI8-U",       N_("Cyrillic, Ukrainian")},
-    {"ISO-8859-7",   N_("Greek")},
-    {"ISO-2022-jp",  N_("Japanese")},
-    {"euc-kr",       N_("Korean")},
-    {"ISO-8859-9",   N_("Turkish")},
-    {"ISO-8859-1",   N_("Western")},
-    {"ISO-8859-15",  N_("Western, New")},
-    {"windows-1252", N_("Western")},
-    {"UTF-8",        N_("Unicode, UTF-8")}
-  };
+  if (_charset.empty())
+      _charset = DEFAULT_CHARSET;
 
-  GtkRadioAction * radio_group (0);
-  for (int i(0), count(G_N_ELEMENTS(charsets)); i<count; ++i)
-  {
-    char * label (g_strdup_printf ("%s (%s)", _(charsets[i].name), charsets[i].charset));
-    char * unique_name (g_strdup_printf ("charset-%s", charsets[i].charset));
-    GtkAction * o ((GtkAction*) g_object_new (GTK_TYPE_RADIO_ACTION,
-      "name", unique_name,
-      "label", label,
-      "value", i,
-      "group", radio_group, NULL));
-    g_object_set_data_full (G_OBJECT(o), "charset", g_strdup(charsets[i].charset), g_free);
-    radio_group = GTK_RADIO_ACTION (o);
-    const bool match (!strcmp (DEFAULT_CHARSET, charsets[i].charset));
-    if (match) {
-      const bool old_val (app_is_toggling_charset_action);
-      app_is_toggling_charset_action = true;
-      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(o), true);
-      app_is_toggling_charset_action = old_val;
-    }
-    g_signal_connect (o, "changed", G_CALLBACK(charset_selected_cb_static), this);
-    gtk_action_group_add_action (_agroup, o);
-    g_object_unref (o);
-
-    const guint id (gtk_ui_manager_new_merge_id (_uim));
-    gtk_ui_manager_add_ui (_uim, id,
-                           "/post/edit-menu/charsets-menu/charset-list",
-                            label, unique_name,
-                            GTK_UI_MANAGER_MENUITEM, false);
-    g_free (unique_name);
-    g_free (label);
-  }
-}
-
-typedef Profiles::strings_t strings_t;
-
-void
-PostUI :: add_editor_list ()
-{
-  // remove the old editor buttons...
-  typedef std::set<guint> unique_uints_t;
-  static unique_uints_t editor_ui_ids;
-  foreach_const (unique_uints_t, editor_ui_ids, it)
-    gtk_ui_manager_remove_ui (_uim, *it);
-
-  // add the new editor buttons...
-  strings_t commands;
-  _profiles.get_editors (commands);
-  const std::string& editor_command (_profiles.get_active_editor ());
-  GtkRadioAction * radio_group (0);
-  for (size_t i=0; i<commands.size(); ++i)
-  {
-    const char * cmd (commands[i].c_str());
-    char name[512], verb[512];
-    g_snprintf (name, sizeof(name), "%s", cmd);
-    g_snprintf (verb, sizeof(verb), "Edit With %s", cmd);
-
-    GtkAction * o = (GtkAction*) g_object_new (GTK_TYPE_RADIO_ACTION,
-       "name", verb,
-       "label", name,
-       "value", i,
-       "group", radio_group, NULL);
-    g_object_set_data_full (G_OBJECT(o), "editor-command", g_strdup(cmd), g_free);
-    radio_group = GTK_RADIO_ACTION (o);
-    const bool match (editor_command == cmd);
-    if (match)
-      gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(o), true);
-    g_signal_connect (o, "changed", G_CALLBACK(editor_selected_cb), this);
-    gtk_action_group_add_action (_agroup, o);
-    g_object_unref (o);
-
-
-    const guint id (gtk_ui_manager_new_merge_id (_uim));
-    gtk_ui_manager_add_ui (_uim, id,
-                           "/post/edit-menu/editors-menu/editor-list",
-                            name, verb,
-                            GTK_UI_MANAGER_MENUITEM, false);
-    editor_ui_ids.insert (id);
-  }
+  char * tmp = e_charset_picker_dialog (_("Character Encoding"),
+                                        _("New Article's Encoding:"),
+                                        _charset.c_str(),
+                                        GTK_WINDOW(root()));
+  set_charset (tmp);
+  free (tmp);
 }
 
 void
@@ -395,32 +273,19 @@ PostUI :: add_actions (GtkWidget * box)
   gtk_action_group_set_translation_domain (_agroup, NULL);
   gtk_action_group_add_actions (_agroup, entries, G_N_ELEMENTS(entries), this);
   gtk_action_group_add_toggle_actions (_agroup, toggle_entries, G_N_ELEMENTS(toggle_entries), this);
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (_agroup, "always-run-editor")),
+                                _prefs.get_flag ("always-run-editor", false));
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (_agroup, "spellcheck")),
+                                _prefs.get_flag ("spellcheck-enabled", DEFAULT_SPELLCHECK_FLAG));
+  gtk_toggle_action_set_active (GTK_TOGGLE_ACTION (gtk_action_group_get_action (_agroup, "wrap")),
+                                _prefs.get_flag ("compose-wrap-enabled", true));
   gtk_ui_manager_insert_action_group (_uim, _agroup, 0);
-
-  add_editor_list ();
-  add_charset_list ();
-}
-
-void
-PostUI :: set_editor_command (const StringView& command)
-{
-  _profiles.set_active_editor (command);
 }
 
 void
 PostUI :: set_charset (const StringView& charset)
 {
-  const std::string new_charset (charset);
-
-  if (_charset != new_charset)
-  {
-    _charset = new_charset;
-
-    std::string unique_name ("charset-");
-    unique_name.append (charset.str, charset.len);
-    GtkAction * action = gtk_action_group_get_action (_agroup, unique_name.c_str());
-    gtk_toggle_action_set_active (GTK_TOGGLE_ACTION(action), true);
-  }
+  _charset = charset;
 }
 
 void
@@ -433,61 +298,6 @@ PostUI :: manage_profiles ()
   apply_profile ();
 }
 
-void
-PostUI :: manage_editors ()
-{
-  GtkWidget * d = gtk_dialog_new_with_buttons (
-    _("Manage Editor List"),
-    GTK_WINDOW(_root),
-    (GtkDialogFlags)(GTK_DIALOG_MODAL|GTK_DIALOG_DESTROY_WITH_PARENT),
-    GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
-    NULL);
-
-  char b[512], lab[512];
-  g_snprintf (b, sizeof(b), _("Editors"));
-  g_snprintf (lab, sizeof(lab), "\t\t\t<b>%s:</b>\t\t\t", b);
-  GtkWidget * w = gtk_label_new (NULL);
-  gtk_label_set_markup (GTK_LABEL(w), lab);
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(d)->vbox), w, false, false, 0);
-  gtk_widget_show (w);
-
-  std::string s;
-  strings_t commands;
-  _profiles.get_editors (commands);
-  foreach_const (strings_t, commands, it)
-    s += *it + "\n";
-
-  GtkWidget * frame = gtk_frame_new (NULL);
-  gtk_frame_set_shadow_type (GTK_FRAME(frame), GTK_SHADOW_IN);
-
-  GtkTextBuffer * buf = gtk_text_buffer_new (NULL);
-  gtk_text_buffer_set_text (buf, s.c_str(), s.size());
-  w = gtk_text_view_new_with_buffer (buf);
-  gtk_container_add (GTK_CONTAINER(frame), w);
-  gtk_box_pack_start_defaults (GTK_BOX(GTK_DIALOG(d)->vbox), frame);
-  gtk_container_set_border_width (GTK_CONTAINER(frame), PAD_BIG);
-  gtk_widget_show_all (frame);
-
-  gtk_dialog_run (GTK_DIALOG (d));
-
-  GtkTextIter start, end;
-  gtk_text_buffer_get_bounds (buf, &start, &end);
-  char * text = gtk_text_buffer_get_text (buf, &start, &end, false);
-  commands.clear ();
-  StringView token, v (text);
-  while (v.pop_token (token, '\n')) {
-    token.trim ();
-    if (!token.empty())
-      commands.push_back (token.to_string());
-  }
-  g_free (text);
-
-  _profiles.set_editors (commands);
-  add_editor_list ();
-
-  gtk_widget_destroy (d);
-}
-
 void
 PostUI :: rot13_selection ()
 {
@@ -927,7 +737,9 @@ PostUI :: spawn_editor ()
   int argc (0);
   char ** argv (0);
   if (ok) {
-    const std::string& editor (_profiles.get_active_editor());
+    std::set<std::string> editors;
+    URL :: get_default_editors (editors);
+    const std::string editor (_prefs.get_string ("editor", *editors.begin()));
     GError * err (0);
     g_shell_parse_argv (editor.c_str(), &argc, &argv, &err);
     if (err != NULL) {
@@ -988,6 +800,8 @@ PostUI :: spawn_editor ()
   ::remove (fname);
   g_free (fname);
   g_strfreev (argv);
+
+  gtk_window_present (GTK_WINDOW(root()));
 }
 
 namespace
@@ -1649,6 +1463,14 @@ namespace
   }
 }
 
+std::string
+PostUI :: utf8ize (const StringView& in) const
+{
+  const char * local_charset = 0;
+  g_get_charset (&local_charset);
+  return content_to_utf8 (in, _charset.c_str(), local_charset);
+}
+
 void
 PostUI :: set_message (GMimeMessage * message)
 {
@@ -1660,21 +1482,22 @@ PostUI :: set_message (GMimeMessage * message)
   _message = message;
 
   // update subject, newsgroups, to fields
-  const char * cpch = g_mime_message_get_subject (message);
-  gtk_entry_set_text (GTK_ENTRY(_subject_entry), cpch ? cpch : "");
+  std::string s = utf8ize (g_mime_message_get_subject (message));
+  gtk_entry_set_text (GTK_ENTRY(_subject_entry), s.c_str());
 
-  cpch = g_mime_message_get_header (message, "Newsgroups");
-  gtk_entry_set_text (GTK_ENTRY(_groups_entry), cpch ? cpch : "");
+  s = utf8ize (g_mime_message_get_header (message, "Newsgroups"));
+  gtk_entry_set_text (GTK_ENTRY(_groups_entry), s.c_str());
 
-  cpch = g_mime_message_get_header (message, "Followup-To");
-  gtk_entry_set_text (GTK_ENTRY(_followupto_entry), cpch ? cpch : "");
+  s = utf8ize (g_mime_message_get_header (message, "Followup-To"));
+  gtk_entry_set_text (GTK_ENTRY(_followupto_entry), s.c_str());
 
-  cpch = g_mime_message_get_header (message, "Reply-To");
-  gtk_entry_set_text (GTK_ENTRY(_replyto_entry), cpch ? cpch : "");
+  s = utf8ize (g_mime_message_get_header (message, "Reply-To"));
+  gtk_entry_set_text (GTK_ENTRY(_replyto_entry), s.c_str());
 
   const InternetAddressList * addresses = g_mime_message_get_recipients (message, GMIME_RECIPIENT_TYPE_TO);
   char * pch  = internet_address_list_to_string (addresses, true);
-  gtk_entry_set_text (GTK_ENTRY(_to_entry), pch ? pch : "");
+  s = utf8ize (pch);
+  gtk_entry_set_text (GTK_ENTRY(_to_entry), s.c_str());
   g_free (pch);
 
   // update 'other headers'
@@ -1682,18 +1505,20 @@ PostUI :: set_message (GMimeMessage * message)
   if (message->mime_part && g_mime_header_has_raw (message->mime_part->headers))
     g_mime_header_foreach (message->mime_part->headers, set_message_foreach_header_func, &data);
   g_mime_header_foreach (GMIME_OBJECT(message)->headers, set_message_foreach_header_func, &data);
-  gtk_text_buffer_set_text (_headers_buf, data.visible_headers.c_str(), -1);
+  s = utf8ize (data.visible_headers);
+  gtk_text_buffer_set_text (_headers_buf, s.c_str(), -1);
   _hidden_headers = data.hidden_headers;
 
   // update body
   int ignored;
   char * tmp = g_mime_message_get_body (message, true, &ignored);
-  if (tmp) {
-    std::string s = TextMassager().fill (tmp);
+  s = utf8ize (tmp);
+  g_free (tmp);
+  if (!s.empty()) {
+    s = TextMassager().fill (s);
     s += "\n\n";
     gtk_text_buffer_set_text (_body_buf, s.c_str(), s.size());
   }
-  g_free (tmp);
 
   // apply the profiles
   update_profile_combobox ();
@@ -1728,6 +1553,11 @@ PostUI :: body_view_realized_cb (GtkWidget * w, gpointer self_gpointer)
   PostUI * self = static_cast<PostUI*>(self_gpointer);
   self->set_message (self->_message);
   self->_unchanged_body = self->get_body ();
+  self->set_wrap_mode (self->_prefs.get_flag ("compose-wrap-enabled", true));
+
+  if (self->_prefs.get_flag ("always-run-editor", false))
+    self->spawn_editor ();
+
   g_signal_handler_disconnect (self->_body_view, self->body_view_realized_handler);
 }
 
@@ -1863,7 +1693,7 @@ PostUI :: create_main_tab ()
   // Mail To
 
   ++row;
-  g_snprintf (buf, sizeof(buf), "<b>%s:</b>", _("_Mail To"));
+  g_snprintf (buf, sizeof(buf), "<b>%s:</b>", _("Mail _To"));
   l = gtk_label_new_with_mnemonic (buf);
   gtk_label_set_use_markup (GTK_LABEL(l), true);
   gtk_misc_set_alignment (GTK_MISC(l), 0.0f, 0.5f);
